-- Handy shortcuts
local wv = BeneCast.WorkVars;
local BeneCastButton = BeneCast.Button;
local BeneCastPanel = BeneCast.Panel;
local BeneCastPanelDropDown = BeneCast.Panel.DropDown;
local hp = BeneCast.Helper;
local L = BeneCast.Localisation;
local const = BeneCast.Constants;

-- Swiped the code from Bongos, thanks Tuller! (off course, he swiped it from Fubar :p)
-- Added documentation, changed it to use BeneCast's setup-table and made it indifferent of the size of the minimap (i hope)

wv.MinimapButton = CreateFrame('Button', 'BeneCastMinimapButton', Minimap)

function wv.MinimapButton:Load()
	self:SetWidth(34);
	self:SetHeight(34);
	self:SetFrameLevel(8);
	self:RegisterForClicks('anyUp');
	self:RegisterForDrag('LeftButton');
	self:SetHighlightTexture('Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight')
	self:SetNormalTexture('Interface\\AddOns\\BeneCast\\BeneCastButton');
	self:SetPushedTexture('Interface\\AddOns\\BeneCast\\BeneCastButtonDown');

	--[[ Not sure if this is needed for my button...
	local overlay = self:CreateTexture(nil, 'OVERLAY')
	overlay:SetWidth(53); overlay:SetHeight(53)
	overlay:SetTexture('Interface/Minimap/MiniMap-TrackingBorder')
	overlay:SetPoint('TOPLEFT')
	]]--

	self:SetScript('OnEnter', self.OnEnter)
	self:SetScript('OnLeave', self.OnLeave)
	self:SetScript('OnClick', self.OnClick)
	self:SetScript('OnDragStart', self.OnDragStart)
	self:SetScript('OnDragStop', self.OnDragStop)
	
	-- The addon will show the button on enable
	self:Hide();
end

function wv.MinimapButton:OnClick(button)
	if button == 'LeftButton' then
		BeneCast.ToggleConfigWindow();
		
	elseif button == 'RightButton' then
		-- something, show a menu? unlock buttons? The sky's the limit! :p
		local pf = BeneCast:GetCurrentProfileID();
		wv.Profiles[pf].unlockbuttons = not wv.Profiles[pf].unlockbuttons;
		BeneCast:Print(wv.Profiles[pf].name .. ' ' .. L['Unlock Buttons'] .. ' = ' .. tostring(wv.Profiles[pf].unlockbuttons));
	end
end

function wv.MinimapButton:OnEnter()
	if not self.dragging then
		GameTooltip:SetOwner(self, 'ANCHOR_TOPRIGHT')
		GameTooltip:SetText(const.Name .. ' ' .. const.Version, 1, 1, 1)
		GameTooltip:Show()
	end
end

function wv.MinimapButton:OnLeave()
	GameTooltip:Hide()
end

function wv.MinimapButton:OnDragStart()
	self.dragging = true
	self:LockHighlight()
	self:SetScript('OnUpdate', self.OnUpdate)
	GameTooltip:Hide()
end

function wv.MinimapButton:OnDragStop()
	self.dragging = nil
	self:SetScript('OnUpdate', nil)
	self:UnlockHighlight()
end

function wv.MinimapButton:SetMinimapButtonPosition(angle)
	wv.Setup.minimappos = angle
end

function wv.MinimapButton:GetMinimapButtonPosition()
	return wv.Setup.minimappos
end

-- While dragging constantly move the button, but keep it on the edge of the minimap
function wv.MinimapButton:OnUpdate()
	local mx, my = Minimap:GetCenter()
	local px, py = GetCursorPosition()
	local scale = Minimap:GetEffectiveScale()

	px, py = px / scale, py / scale

	-- First save the angle of the current cursor position compared to the center of the minimap
	self:SetMinimapButtonPosition(math.deg(math.atan2(py - my, px - mx)) % 360)
	-- Then have the button position itself along the edge at that angle
	self:UpdatePosition()
end

-- magic fubar code for updating the minimap button's position to have it stick to the edge
function wv.MinimapButton:UpdatePosition()
	local angle = math.rad(self:GetMinimapButtonPosition())
	local cos = math.cos(angle)
	local sin = math.sin(angle)
	local minimapShape = GetMinimapShape and GetMinimapShape() or 'ROUND'
	local w, h = Minimap:GetWidth()+12, Minimap:GetHeight()+10;

	-- The code supports 5 shapes: circle, full square, one corner is right angled, two corners right angled and three corners right-angled
	-- We must determine which corner we are at
	local round = false
	if minimapShape == 'ROUND' then
		round = true
	elseif minimapShape == 'SQUARE' then
		round = false
	elseif minimapShape == 'CORNER-TOPRIGHT' then
		round = not(cos < 0 or sin < 0)
	elseif minimapShape == 'CORNER-TOPLEFT' then
		round = not(cos > 0 or sin < 0)
	elseif minimapShape == 'CORNER-BOTTOMRIGHT' then
		round = not(cos < 0 or sin > 0)
	elseif minimapShape == 'CORNER-BOTTOMLEFT' then
		round = not(cos > 0 or sin > 0)
	elseif minimapShape == 'SIDE-LEFT' then
		round = cos <= 0
	elseif minimapShape == 'SIDE-RIGHT' then
		round = cos >= 0
	elseif minimapShape == 'SIDE-TOP' then
		round = sin <= 0
	elseif minimapShape == 'SIDE-BOTTOM' then
		round = sin >= 0
	elseif minimapShape == 'TRICORNER-TOPRIGHT' then
		round = not(cos < 0 and sin > 0)
	elseif minimapShape == 'TRICORNER-TOPLEFT' then
		round = not(cos > 0 and sin > 0)
	elseif minimapShape == 'TRICORNER-BOTTOMRIGHT' then
		round = not(cos < 0 and sin < 0)
	elseif minimapShape == 'TRICORNER-BOTTOMLEFT' then
		round = not(cos > 0 and sin < 0)
	end

	-- Depending on the shape the corner is round or right-angled
	local x, y
	if round then
		x = cos*(w/2)
		y = sin*(h/2)
	else
		x = math.max(-(w/2)-2, math.min(110*cos, (w/2)+4 ))
		y = math.max(-(h/2)-6, math.min(110*sin, (h/2)+2 ))
	end

	-- Position the button
	self:SetPoint('CENTER', x, y)
end

wv.MinimapButton:Load()