﻿-- Handy shortcuts
local wv = BeneCast.WorkVars;
local BeneCastButton = BeneCast.Button;
local BeneCastPanel = BeneCast.Panel;
local BeneCastPanelDropDown = BeneCast.Panel.DropDown;
local hp = BeneCast.Helper;
local L = BeneCast.Localisation;
local const = BeneCast.Constants;

BeneCast.Timers = {
	UpdatePanels = {
		period = 1,
	},
	CheckEnchants = {
		period = 0.5,
	},
	FlashButtons = {
		period = 0.2,
	},
	CheckRange = {
		period = 0.5,
	},
};

local function Timer_Disable(self)
	if self.handle ~= nil then
		BeneCast:CancelTimer(self.handle);
		self.handle = nil;
	end
end

local function Timer_Enable(self)
	if self.handle == nil then
		self.handle = BeneCast:ScheduleRepeatingTimer(self.Event, self.period, self);
	end
end

local function Timer_Toggle(self)
	if self.handle == nil then
		self:Enable();
	else
		self:Disable();
	end
end

for timername, timer in pairs(BeneCast.Timers) do
	BeneCast.Timers[timername].Enable = Timer_Enable;
	BeneCast.Timers[timername].Disable = Timer_Disable;
	BeneCast.Timers[timername].Toggle = Timer_Toggle;
end

function BeneCast.Timers.UpdatePanels:Event()

	for unit, when in pairs(wv.UnitsToUpdate) do
		if unit ~= 'count' and (GetTime() > when) then
			wv.UnitsToUpdate[unit] = nil;
			wv.UnitsToUpdate.count = wv.UnitsToUpdate.count - 1;
			BeneCast:UpdatePanelsForUnit(unit);
		end
	end
	
	if wv.UnitsToUpdate.count == 0 then
		self:Disable();
	end

end

-- Function to return the spellinfo for the temp enchant on a weapon slot
local function CheckForEnchant(slotname)

	local slotid, line, spellinfo;
	slotid = GetInventorySlotInfo(slotname);
	
	-- Set the tooltip to the slot
	BeneCast_Tooltip:ClearLines();
	BeneCast_Tooltip:SetInventoryItem('player', slotid);
	
	for i = 1, BeneCast_Tooltip:NumLines() do
		line = getglobal('BeneCast_TooltipTextLeft' .. i):GetText();
		if line then
			for name, group in pairs(wv.TempEnchantsInfo) do
				if string.find(name, line) then -- Like finding 'Flametongue' in 'Flametongue Weapon'
					spellinfo = wv.SpellInfo[group];
					if spellinfo then
						return hp._CopyTable(spellinfo,true);
					end
				end
			end
		end
	end
	
	return nil;
	
end

function BeneCast.Timers.CheckEnchants:Event()

	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges;
	hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo();
	
	-- Set the tooltip allright
	if not BeneCast_Tooltip:IsOwned(WorldFrame) then
		BeneCast_Tooltip:SetOwner(WorldFrame, 'ANCHOR_NONE');
	end
	
	local oldname, updatepanel = false;
	if ( hasMainHandEnchant ) then
		oldname = nil;
		if wv.MHWeaponEnchant then
			oldname = wv.MHWeaponEnchant.name;
		end
		wv.MHWeaponEnchant = CheckForEnchant('MainHandSlot');
		if wv.MHWeaponEnchant then
			wv.MHWeaponEnchant.duration = mainHandExpiration;
		end
		if wv.MHWeaponEnchant and wv.MHWeaponEnchant.name ~= oldname then
			updatepanel = true;
		end
	end
	if ( hasOffHandEnchant ) then
		oldname = nil;
		if wv.OHWeaponEnchant then
			oldname = wv.OHWeaponEnchant.name;
		end
		wv.OHWeaponEnchant = CheckForEnchant('SecondaryHandSlot');
		if wv.OHWeaponEnchant then
			wv.OHWeaponEnchant.duration = offHandExpiration;
		end
		if wv.OHWeaponEnchant and wv.OHWeaponEnchant.name ~= oldname then
			updatepanel = true;
		end
	end
	if updatepanel then
		BeneCast:UpdatePanelsForUnit('player');
	end

end

function BeneCast.Timers.FlashButtons:Event()

	-- Update all button-flashing at once
	local buffAlphaValue = nil;
	local buttonsintable = 0;
	for pfid, pfunits in pairs(wv.ButtonsThatFlash) do
		for unit, unitbuttons in pairs(pfunits) do
			for k, button in pairs(unitbuttons) do
				if buffAlphaValue == nil then --this way it only has to be calculated once
					if ( BuffFrameFlashState == 1 ) then
						buffAlphaValue = (BUFF_FLASH_TIME_ON - BuffFrameFlashTime) / BUFF_FLASH_TIME_ON;
						buffAlphaValue = buffAlphaValue * (1 - BUFF_MIN_ALPHA) + BUFF_MIN_ALPHA;
					else
						buffAlphaValue = BuffFrameFlashTime / BUFF_FLASH_TIME_ON;
						buffAlphaValue = (buffAlphaValue * (1 - BUFF_MIN_ALPHA)) + BUFF_MIN_ALPHA;
					end
				end
				button:SetAlpha(buffAlphaValue);
				buttonsintable = buttonsintable + 1;
			end
		end
	end
	if buttonsintable == 0 then
		self:Disable();
	end

end

function BeneCast.Timers.CheckRange:Event()

	local inrange, pf, button, spellinfoid, range;
	pf = BeneCast:GetCurrentProfileID();
	
	if not wv.Panels[pf] then
		-- No panels yet
		return;
	end
	
	for unit, panel in pairs(wv.Panels[pf]) do
		if unit ~= 'player' then
			if panel:IsVisible() and UnitExists(unit) then
				for i, button in pairs(panel.buttons) do
					if button.action and button.action.origin == 'spell' then
						spellinfoid = button.action.spellbookid;
						range = button.Range;
						inrange = IsSpellInRange(spellinfoid, unit);
						if (inrange == nil) or (inrange == 0) then
							range:Show();
							if unit == 'target' then
								BeneCast:Debug(L['Spell %1 out of range for %2'](button.action.name,unit));
							end
						else
							range:Hide();
						end
					end
				end
			end
		end
	end

end